#
# See the file "LICENSE" for the full license governing this code.
#

require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'puppet_x/enterprisemodules/mq/core'

Puppet::Type.newtype(:mq_certificate) do
  include PuppetX::EnterpriseModules::Mq::Type
  def self.module_name
    'mq_config'
  end

  desc <<-DESC
  This resource allows you to manage SSL Certificates for MQ in a database.

  It allows multiple operations:

  1) Adding a certificate from an ascii of binary file
  2) Importing a labeled certificate from an other certificate database
  3) Creating a self signed certificate


  ## Option 1: Adding a certificate from an ascii of binary file

  Puppet uses this action when the `source` parameter is specified and the specified `source_type` is either
  `ascii` or `binary`. Here is an example on how to use this:

      mq_certificate {'/var/mqm/keys/test_db_2.kdb:my_label_1':
        ensure          => 'present',
        password        => 'verysecret',
        source          => '/var/mqm/keys/my_label_2.ascii',
        source_type     => 'ascii',
      }

  ## Option 2: Importing a labeled certificate from an other certificate database

  Puppet uses this action when the `source` parameter is specified and the specified `source_type` is:

  - cms
  - kdb
  - pkcs12
  - p12
  - pkcs7
  - pem

  Here is an example on how to use this:

      mq_certificate {'/var/mqm/keys/test_db_2.kdb:my_label_1':
        ensure          => 'present',
        password        => 'verysecret',
        source          => '/var/mqm/keys/my_label_1.p12',
        source_type     => 'p12',
        source_label    => 'my_label_1',
        source_password => 'verysecret',
      }

  ## Option 3: Creating a self signed certificate.

  When no source value is given, Puppet creates a self signed cerificate withe the specified label inside
  of the certificate database. Here is an example on how that works:

      mq_certificate {'/var/mqm/keys/test_db_1.kdb:my_label_1':
        ensure             => 'present',
        password           => 'verysecret',
        distinguished_name => 'cn=bert.....',
      }

  DESC

  ensurable

  map_titles_to_attributes([/((.*):(.*))/, [:name, :file_name, :label]])

  to_get_raw_resources do
    []
  end

  on_prefetch do |resources, provider_class|
    resources.each do |name, properties|
      file_name = (properties[:file_name]).to_s
      next unless File.exist?(file_name)

      pw_options = if properties[:password]
                     "-pw #{properties[:password]}"
                   else
                     '-stashed'
                   end

      cmd_output = runmqakm("-cert -list all -db #{file_name} #{pw_options}")
      Puppet.debug cmd_output
      next if cmd_output == "No certificates were found.\n"

      exists = cmd_output.split("\n")[2..-1].map { |e| e.split("\t")[1..-1].join }.map(&:strip).any?(properties[:label])
      next unless exists

      resource = { name => properties }.merge(:ensure => :present)
      resources[name].provider = provider_class.new(resource)
    end
  end

  on_create do
    parameters[:source].process if parameters[:source]
    if !source
      create_certificate
    elsif %w[ascii binary pem].include? source_type.to_s
      add_certificate
    else
      import_certificate
    end
  end

  on_modify do
    fail "Modification of key database #{name} is not supported."
  end

  on_destroy do
    options = []
    options << "-pw #{password}" if password
    Puppet.debug runmqakm("-cert -delete -db #{file_name} -label #{label} #{options.join(' ')}")
  end
  def create_certificate
    options = []
    options << "-pw #{password}" if password
    options << "-size #{key_size}" if key_size
    options << "-x509version #{x509version}" if x509version
    options << "-expire #{expire_days}" if expire_days
    Puppet.debug runmqakm("-cert -create -db #{file_name} -label #{label} -dn #{distinguished_name} #{options.join(' ')}")
  end

  def import_certificate
    options = []
    options << if source_password
                 "-pw #{source_password}"
               else
                 '-stashed'
               end
    options << "-type #{db_type}" if db_type
    options << if password
                 "-target_pw #{password}"
               else
                 '-target_stashed'
               end
    options << "-new_label #{label}"
    options << if source_label
                 "-label #{source_label}"
               else
                 "-label #{label}"
               end
    Puppet.debug runmqakm("-cert -import -target #{file_name} -file #{source} #{options.join(' ')}")
  end

  def add_certificate
    options = []
    options << if password
                 "-pw #{password}"
               else
                 '-stashed'
               end
    options << if source_type == :pem
                 '-type pem'
               elsif db_type
                 "-type #{db_type}"
               end
    options << "-format #{source_type}" if %w[ascii binary].include? source_type.to_s
    # We use the strange option to access the trust parameter because the normal
    # way is shadowed by a puppet variable
    options << "-trust #{to_hash[:trust]}" if to_hash[:trust]
    Puppet.debug runmqakm("-cert -add -db #{file_name} -label #{label} -file #{source} #{options.join(' ')}")
  end

  parameter :name
  parameter :file_name
  parameter :label
  parameter :password
  #
  # Generic
  #
  parameter :source
  parameter :tmp_dir
  #
  # For importing and adding certificates
  #
  parameter :db_type
  parameter :source_password
  parameter :source_label
  parameter :source_type
  parameter :trust
  #
  # For generating self signed certificates
  #
  parameter :distinguished_name
  parameter :key_size
  parameter :x509version
  parameter :expire_days
end
