#
# See the file "LICENSE" for the full license governing this code.
#

require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'puppet_x/enterprisemodules/mq/core'

Puppet::Type.newtype(:mq_certificate_file) do
  include PuppetX::EnterpriseModules::Mq::Type
  def self.module_name
    'mq_config'
  end

  desc <<-DESC
  Extracts individual certificates from a certificate key database.

  Depending on the value f the `type` parameter, different actions will be done.

  - When the type is either `ascii` of `binary` a certificate file will be created
  - When the type is one of: `cms`,`kdb`,`pkcs12`,`p12`,`pkcs7`,`pem` a certificate
    database of the specified type will be created and the certificate with the specified
    label will be added to it.

  Here is an example on how you can use this:

      mq_certificate_file {'/var/mqm/keys/my_label_1.p12':
        ensure          => 'present',
        ource_type      => 'p12',
        key_db          => '/var/mqm/keys/test_db_1.kdb',
        label           => 'my_label_1',
        type            => 'p12',
        password        => 'verysecret',
        target_password => 'verysecret',
      }

  DESC

  ensurable

  to_get_raw_resources do
    []
  end

  on_prefetch do |resources, provider_class|
    resources.each do |name, properties|
      next unless File.exist?(name.to_s)

      resource = { name => properties }.merge(:ensure => :present)
      resources[name].provider = provider_class.new(resource)
    end
  end

  on_create do
    fail "#{path}: parameter 'type' must be specified." if self[:type].nil?

    if %w[ascii binary].include? self[:type].to_s
      extract_certificate
    else
      export_certificate
    end
  end
  def extract_certificate
    options = []
    options << "-pw #{password}" if password
    options << "-type #{db_type}" if db_type
    options << "-format #{self[:type]}" if self[:type]
    Puppet.debug runmqakm("-cert -extract -db #{key_db} -label #{label} -target #{name} #{options.join(' ')}")
  end

  def export_certificate
    options = []
    options << "-pw #{password}" if password
    options << "-type #{db_type}" if db_type
    options << "-target_type #{self[:type]}" if self[:type]
    options << "-target_pw #{target_password}" if target_password
    Puppet.debug runmqakm("-cert -export -db #{key_db} -label #{label} -target #{name} #{options.join(' ')}")
  end

  on_modify do
    fail "Modification of certificate file #{name} is not supported."
  end

  on_destroy do
    File.delete(name)
  end

  parameter :name
  parameter :key_db
  parameter :type
  parameter :label
  parameter :password
  parameter :target_password
  parameter :db_type
end
