#
# See the file "LICENSE" for the full license governing this code.
#

require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'puppet_x/enterprisemodules/mq/core'

Puppet::Type.newtype(:mq_channel) do
  include PuppetX::EnterpriseModules::Mq::Type
  def self.module_name
    'mq_config'
  end

  desc <<-DESC
  This resource allows you to manage a MQ Channel.

  Here is an example on how you should use this with the CHLTYPE in the name:

        mq_channel { 'QM1/RCVR/EM_TEST_CHANNEL':
          ensure   => 'present',
          comphdr  => 'NONE',
          compmsg  => 'NONE',
          hbint    => '320',
          kaint    => 'AUTO',
          maxmsgl  => '4194304',
          monchl   => 'QMGR',
          npmspeed => 'FAST',
          seqwrap  => '999999999',
          sslcauth => 'REQUIRED',
          statchl  => 'QMGR',
          trptype  => 'TCP',
          usedlq   => 'YES',
        }

  Here is an deprecated example on how you should use this:

        mq_channel { 'QM1/EM_TEST_CHANNEL':
          ensure   => 'present',
          chltype  => 'RCVR',
          comphdr  => 'NONE',
          compmsg  => 'NONE',
          hbint    => '320',
          kaint    => 'AUTO',
          maxmsgl  => '4194304',
          monchl   => 'QMGR',
          npmspeed => 'FAST',
          seqwrap  => '999999999',
          sslcauth => 'REQUIRED',
          statchl  => 'QMGR',
          trptype  => 'TCP',
          usedlq   => 'YES',
        }
  DESC

  ensurable

  map_titles_to_attributes([
                             %r{((.*)/(.*)/(.*))}, [:name, :qmgr, :chltype, :channel],
                             %r{((.*)/(.*))}, [:name, :qmgr, :channel]
                           ])

  to_get_raw_resources do
    channels = []
    running_queue_managers.collect do |qmgr, status|
      next unless status == 'Running'

      channels_on(qmgr).collect do |channel, chltype|
        output = runmqsc("display channel('#{channel}') where (CHLTYPE EQ #{chltype}) ALL", :qmgr => qmgr)
        channels << { :name => name, :chltype => chltype, :channel => channel, :qmgr => qmgr, :output => output }
      end
    end
    channels
  end
  validate do
    return if self[:ensure] == :absent
    fail 'property chltype is required' if chltype.nil?
  end

  on_create do
    when_running { [:runmqsc, "define channel('#{channel}') chltype(#{chltype})", { :qmgr => qmgr }] }
  end

  on_modify do
    when_running { [:runmqsc, "alter channel('#{channel}') chltype(#{chltype})", { :qmgr => qmgr }] }
  end

  on_destroy do
    when_running do
      if chltype.to_s == 'CLNTCONN'
        [:runmqsc, "delete channel('#{channel}') chltable(clnttbl)", { :qmgr => qmgr }]
      else
        [:runmqsc, "delete channel('#{channel}')", { :qmgr => qmgr }]
      end
    end
  end

  parameter :name
  parameter :qmgr
  parameter :channel
  parameter :chltype

  property :conname
  property :affinity
  property :amqpka
  property :backlog
  property :batchhb
  property :batchint
  property :batchlim
  property :batchsz
  property :certlabl
  property :clntwght
  property :clusnl
  property :cluster
  property :clwlprty
  property :clwlrank
  property :clwlwght
  property :comphdr
  property :compmsg
  property :convert
  property :defcdisp
  property :defrecon
  property :descr
  property :discint
  property :hbint
  property :kaint
  property :like
  property :locladdr
  property :longrty
  property :longtmr
  property :maxinst
  property :maxinstc
  property :maxmsgl
  property :mcaname
  property :mcatype
  property :mcauser
  property :modename
  property :monchl
  property :mrdata
  property :mrexit
  property :mrrty
  property :mrtmr
  property :msgdata
  property :msgexit
  property :netprty
  property :npmspeed
  property :password
  property :port
  property :propctl
  property :putaut
  property :qmname
  property :rcvdata
  property :rcvexit
  property :replace
  property :scydata
  property :scyexit
  property :senddata
  property :sendexit
  property :seqwrap
  property :sharecnv
  property :shortrty
  property :shorttmr
  property :sslcauth
  property :sslciph
  property :sslpeer
  property :statchl
  property :tpname
  property :tproot
  property :trptype
  property :usecltid
  property :usedlq
  property :userid
  property :xmitq
end
