#
# See the file "LICENSE" for the full license governing this code.
#

require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'puppet_x/enterprisemodules/mq/core'

Puppet::Type.newtype(:mq_channel_authentication) do
  include PuppetX::EnterpriseModules::Mq::Type
  def self.module_name
    'mq_config'
  end

  desc <<-DESC
  This resource allows you to manage a MQ Channel authenticatio record.

  Here is an example on how you should use this:

        mq_channel_authentication { 'QM1/EM_TEST_CHANNEL':
          ensure   => 'present',
            ...
        }

  When used for `ADDRESMAP` records the syntax is:

      mq_channel_authentication { 'QM1/ADRESSMAP/ADMIN/10.2.248.*':
        ensure   => 'present',
        mcauser  => 'my_user',
      }


  DESC

  ensurable

  map_titles_to_attributes([
                             %r{((.*)/(.*)/(.*)/(.*))}, [:name, :qmgr, :record_type, :channel_name, :address],
                             %r{((.*)/(.*)/(.*))}, [:name, :qmgr, :record_type, :channel_name]
                           ])

  to_get_raw_resources do
    channel_authentication_records = []
    running_queue_managers.collect do |qmgr, status|
      next unless status == 'Running'

      channel_authentication_records_on(qmgr).collect do |name, type, address, output|
        channel_authentication_records << { :name => name, :type => type, :address => address, :qmgr => qmgr, :output => output }
      end
    end
    channel_authentication_records
  end

  on_create do
    when_running do
      if record_type.to_s == 'ADDRESSMAP'
        [:runmqsc, "set chlauth('#{channel_name}') type(#{record_type}) address(#{address})", { :qmgr => qmgr }]
      else
        [:runmqsc, "set chlauth('#{channel_name}') type(#{record_type})", { :qmgr => qmgr }]
      end
    end
  end

  on_modify do
    when_running do
      #
      # Somehow the action(change) is not consistent. Therefore we first remove the existing record
      # and then re-add the new record
      #
      delete_record
      if record_type.to_s == 'ADDRESSMAP'
        [:runmqsc, "set chlauth('#{channel_name}') type(#{record_type}) address(#{address})", { :qmgr => qmgr }]
      else
        [:runmqsc, "set chlauth('#{channel_name}') type(#{record_type}) action(add)", { :qmgr => qmgr }]
      end
    end
  end

  on_destroy do
    when_running do
      if record_type.to_s == 'ADDRESSMAP'
        [:runmqsc, "set chlauth('#{channel_name}') type(#{record_type}) address(#{address}) action(remove)", { :qmgr => qmgr }]
      else
        [:runmqsc, "set chlauth('#{channel_name}') type(#{record_type}) action(removeall)", { :qmgr => qmgr }]
      end
    end
  end

  after_modify do
    when_running { update_security }
  end

  after_create do
    when_running { update_security }
  end

  def update_security
    Puppet.debug('refreshing security after change.')
    runmqsc('refresh security type(authserv)', :qmgr => qmgr)
  end

  def delete_record
    runmqsc("set chlauth('#{channel_name}') type(#{record_type}) action(removeall)", :qmgr => qmgr)
  end

  parameter :name
  parameter :qmgr
  parameter :channel_name
  parameter :record_type
  parameter :address

  group(:all) do # Always use all properties to construct commands
    property :sslpeer
    property :sslcerti
    property :clntuser
    property :chckclnt
    property :qmname
    property :addrlist
    property :userlist
    property :usersrc
    property :mcauser
    property :custom
    property :warn
    property :descr
  end
end
