#
# See the file "LICENSE" for the full license governing this code.
#

require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'puppet_x/enterprisemodules/mq/core'

Puppet::Type.newtype(:mq_manager) do
  include PuppetX::EnterpriseModules::Mq::Type
  def self.module_name
    'mq_config'
  end
  include EasyType
  desc <<-DESC
  This resource allows you to manage a MQ queue manager.

  Here is an example on how you should use this:

      mq_manager { 'QM1':
        ensure   => 'present',
        acctcono => 'DISABLED',
        acctint  => '1800',
        acctmqi  => 'OFF',
        acctq    => 'OFF',
        activrec => 'MSG',
        actvcono => 'DISABLED',
        actvtrc  => 'OFF',
        authorev => 'DISABLED',
        ccsid    => '1208',
        certlabl => 'ibmwebspheremqqm1',
        certvpol => 'ANY',
        chad     => 'DISABLED',
        chadev   => 'DISABLED',
        chlauth  => 'ENABLED',
        chlev    => 'DISABLED',
        clwllen  => '100',
        clwlmruc => '999999999',
        clwluseq => 'LOCAL',
        cmdev    => 'DISABLED',
        configev => 'DISABLED',
        connauth => 'SYSTEM.DEFAULT.AUTHINFO.IDPWOS',
        defclxq  => 'SCTQ',
        inhibtev => 'DISABLED',
        ipaddrv  => 'IPV4',
        localev  => 'DISABLED',
        loggerev => 'DISABLED',
        markint  => '5000',
        maxhands => '256',
        maxmsgl  => '4194304',
        maxpropl => 'NOLIMIT',
        maxumsgs => '10000',
        monacls  => 'QMGR',
        monchl   => 'OFF',
        monq     => 'OFF',
        perfmev  => 'DISABLED',
        psclus   => 'ENABLED',
        psmode   => 'ENABLED',
        psnpmsg  => 'DISCARD',
        psnpres  => 'NORMAL',
        psrtycnt => '5',
        pssyncpt => 'IFPER',
        remoteev => 'DISABLED',
        revdns   => 'ENABLED',
        routerec => 'MSG',
        schinit  => 'QMGR',
        scmdserv => 'QMGR',
        sslev    => 'DISABLED',
        sslfips  => 'NO',
        sslkeyr  => '/var/mqm/qmgrs/QM1/ssl/key',
        sslrkeyc => '0',
        statacls => 'QMGR',
        statchl  => 'OFF',
        statint  => '1800',
        statmqi  => 'OFF',
        statq    => 'OFF',
        status   => 'Running',
        strstpev => 'ENABLED',
        suiteb   => 'NONE',
        treelife => '1800',
        trigint  => '999999999',
      }

  DESC

  ensurable

  to_get_raw_resources do
    information = queue_managers.collect do |qmgr, status|
      output = status == 'Running' ? runmqsc('display qmgr', :qmgr => qmgr) : ''
      [qmgr, output, status]
    end
    information.collect { |e| [:name, :output, :status].zip(e).to_h }
  end
  on_create do
    if standby?
      addmqinf("-s QueueManager -v Name='#{name}' -v Directory='#{name}' -v Prefix=/var/mqm -v DataPath='#{datapath}/#{sanitized_name}'")
      start_qm
      nil # Ignore the properties, They are set on the primary Queue Manager
    else
      options = []
      options << "-ld '#{logpath}'" if logpath
      options << "-md '#{datapath}'" if datapath
      options << "-lf #{logfile_pages}" if logfile_pages
      case logging
      when nil, ''
      # Nothing
      when :circular
        options << '-lc'
      when :linear
        options << '-ll'
      when :linear_automatic
        options << '-lla'
      when :linear_archiving
        options << '-lln'
      else
        fail "internal error: invalid logging value '#{logging}' found"
      end
      options << '-q ' if default.to_s == 'true'

      crtmqm("#{options.join(' ')} #{name}")
      return nil if status.to_s.include?('End')

      start_qm
      [:runmqsc, 'alter qmgr', { :qmgr => name }]
    end
  end

  on_modify do
    #
    # Only do something when the requested status is running.
    # If the requested state is stopped, we are already stopped
    # at this point in time.
    #
    return nil if status.to_s.include?('End')

    [:runmqsc, 'alter qmgr', { :qmgr => name, :skip_without_properties => true }]
  end

  on_destroy do
    if provider.status.to_s.include?('Running') # First stop it, if it is still running
      stop_qm
      sleep 2
    end
    if provider.status.to_s.include?('standby')
      rmvmqinf("-s QueueManager '#{name}'")
      nil # Ignore the properties, They are set on the primary Queue Manager
    else
      [:dltmqm, name, {}] # Then delete it
    end
  end

  def standby?
    standby.to_s == 'true'
  end

  def start_qm
    if running_in_systemd?
      Puppet::Util::Execution.execute("systemctl start mq@#{name}")
    else
      # When the queue manager is running on the other node of
      # a multi instance setup, strqm returns:
      #     A standby instance of queue manager 'MQBRSP02' has been started. The active instance is running elsewhere.
      # The exit status is 30 and is valid
      multi_instance ? strmqm("-x #{name}", :allowed_exits => [30]) : strmqm(name)
    end
  end

  def stop_qm
    if running_in_systemd?
      Puppet::Util::Execution.execute("systemctl stop mq@#{name}")
    else
      provider.status.to_s.include?('standby') ? endmqm("-x #{name}") : endmqm("-p #{name}")
    end
  end

  def running_in_systemd?
    if Facter.value(:kernel) == 'Linux'
      `systemctl list-units --state running | grep mq@#{name}` != ''
    else
      false
    end
  end

  def sanitized_name
    #
    # When the name contains non standard characters like for example a '.', MQ changes these for a '!'
    # in the path name it generates.
    #
    @sanitized_name ||= name.gsub(/\W/, '!')
  end

  parameter :name
  parameter :logpath
  parameter :logging
  parameter :datapath
  parameter :multi_instance
  parameter :standby
  parameter :default
  parameter :logfile_pages

  property :status

  property :acctcono
  property :acctint
  property :acctmqi
  property :acctq
  property :activrec
  property :actvcono
  property :actvtrc
  property :adoptmca
  property :authorev
  property :ccsid
  property :certlabl
  property :certvpol
  property :chadev
  property :chadexit
  property :chad
  property :chlauth
  property :chlev
  property :clwldata
  property :clwlexit
  property :clwllen
  property :clwlmruc
  property :clwluseq
  property :cmdev
  property :configev
  property :connauth
  property :deadq
  property :defclxq
  property :defxmitq
  property :descr
  property :expryint
  property :groupur
  property :igqaut
  property :igq
  property :igquser
  property :imgintvl
  property :imglogln
  property :imgrcovo
  property :imgrcovq
  property :imgsched
  property :inhibtev
  property :ipaddrv
  property :localev
  property :loggerev
  property :luname
  property :markint
  property :maxhands
  property :maxmsgl
  property :maxpropl
  property :maxumsgs
  property :monacls
  property :monchl
  property :monq
  property :parent
  property :perfmev
  property :psclus
  property :psmode
  property :psnpmsg
  property :psnpres
  property :psrtycnt
  property :pssyncpt
  property :remoteev
  property :reposnl
  property :repos
  property :revdns
  property :routerec
  property :schinit
  property :scmdserv
  property :scycase
  property :sqqmname
  property :sslcrlnl
  property :sslcryp
  property :sslev
  property :sslfips
  property :sslkeyr
  property :sslrkeyc
  property :ssltasks
  property :statacls
  property :statchl
  property :statint
  property :statmqi
  property :statq
  property :strstpev
  property :suiteb
  property :treelife
  property :trigint
end
