#
# See the file "LICENSE" for the full license governing this code.
#
module PuppetX
  module EnterpriseModules
    module Mq
      # docs
      class Command
        # rubocop: disable Lint/OrAssignmentToConstant
        DEFAULT_TIMEOUT ||= 300 # 5 minutes

        VALID_OPTIONS ||= [
          :os_user,
          :password,
          :allowed_exits,
          :timeout
        ].freeze
        # rubocop: enable Lint/OrAssignmentToConstant

        attr_reader(*VALID_OPTIONS)

        def initialize(command, options, valid_options = VALID_OPTIONS)
          @valid_options = valid_options
          check_options(options)
          kernel = Facter.value(:kernel)
          default_user = case kernel
                         when 'windows'
                           'MUSR_MQADMIN'
                         when 'Linux', 'AIX', 'SunOS'
                           'mqm'
                         else
                           fail "#{kernel} is not supported at this point in time."
                         end
          @mq_path = case kernel
                     when 'windows'
                       nil # Windows has it's own path
                     when 'Linux', 'SunOS'
                       '/opt/mqm/bin'
                     when 'AIX'
                       '/usr/mqm/bin'
                     else
                       fail "#{kernel} is not supported at this point in time."
                     end
          @command = command
          @password = options[:password] # may be empty
          @timeout = options.fetch(:timeout) { DEFAULT_TIMEOUT }
          @os_user = options.fetch(:os_user) { default_user }
          @allowed_exits = options.fetch(:allowed_exits) { [] }
        end

        def command_string(arguments = '')
          kernel = Facter.value(:kernel)
          case kernel
          when 'windows'
            "\"C:\\Program Files\\IBM\\MQ\\bin\\setmqenv.cmd\" -s &&  #{@command} #{arguments}"
          when 'Linux', 'AIX', 'SunOS'
            ". #{@mq_path}/setmqenv -s; #{@command} #{arguments}"
          else
            fail "#{kernel} is not supported at this point in time."
          end
        end

        def execute(arguments)
          options = { :failonfail => true, :combine => true, :uid => @os_user }
          value = ''
          within_time(@timeout) do
            Puppet.debug "Executing #{@command} command: #{arguments} "
            begin
              value = Puppet::Util::Execution.execute(command_string(arguments), options)
            rescue Puppet::ExecutionFailure => e
              exit_status = e.message.scan(/returned (\d+):/).first.first.to_i
              raise unless @allowed_exits.include?(exit_status)

              Puppet.debug "Allowed e #{e.message}"
            end
          end
          value
        end

        private

        def within_time(timeout, &block)
          Puppet.debug "Using timeout #{timeout}"
          if timeout.zero?
            yield
          else
            Timeout.timeout(timeout, &block)
          end
        end

        def check_options(options)
          options.each_key { |key| raise ArgumentError, "option #{key} invalid for #{@command}. Only #{@valid_options.join(', ')} are supported" unless @valid_options.include?(key) }
        end
      end
    end
  end
end
