#
# See the file "LICENSE" for the full license governing this code.
#
module Puppet_X
  # Docs
  module EnterpriseModules
    # Docs
    module Mq
      # encoding: UTF-8
      #
      #
      # Define all common mungers available for all types
      #
      module Mungers
        #
        # Define some IBM MQ specific mungers.
        #
        module QueueName
          #
          # This munger checks if the mq_queue name conformst to new standard
          # containging the qtype in name. Like:
          #
          # QMGR1/QREMOTE/MY.FIRST.QUEUE
          #
          # If it is old name, like:
          #
          # QMGR1/MY.FIRST.QUEUE
          #
          # Then it adds the queue QTYPE to name from the manifest or raises
          # error.
          #
          def unsafe_munge(original_value)
            return original_value if original_value.scan('/').length == 2

            qmgr, _, queue = original_value.partition('/')
            qtype = resource.original_parameters[:qtype]
            raise "\nPlease provide the qtype for #{qmgr}/#{queue}" unless qtype

            "#{qmgr}/#{qtype}/#{queue}"
          end
        end

        # Docs
        module TopicName
          #
          # This munger checks if the mq_topic name conformst to new standard
          # containging the qtype in name. Like:
          #
          # QMGR1/LOCAL/MY.FIRST.TOPIC
          #
          # If it is old name, like:
          #
          # QMGR1/MY.FIRST.TOPIC
          #
          # Then it adds the queue TTYPE to name from the manifest or raises
          # error.
          #
          def unsafe_munge(original_value)
            return original_value if original_value.scan('/').length == 2

            qmgr, _, topic = original_value.partition('/')
            ttype = resource.original_parameters[:ttype]
            raise "\nPlease provide the qtype for #{qmgr}/#{topic}" unless ttype

            "#{qmgr}/#{ttype}/#{topic}"
          end
        end

        # Docs
        module ChannelName
          #
          # This munger checks if the mq_channel name conformst to new standard
          # containging the qtype in name. Like:
          #
          # QMGR1/RCVR/MY.FIRST.CHANNEL
          #
          # If it is old name, like:
          #
          # QMGR1/MY.FIRST.CHANNEL
          #
          # Then it adds the queue CHLTYPE to name from the manifest or raises
          # error.
          #
          def unsafe_munge(original_value)
            return original_value if original_value.scan('/').length == 2

            qmgr, _, channel = original_value.partition('/')
            chltype = resource.original_parameters[:chltype]
            raise "\nPlease provide the chltype for #{qmgr}/#{channel}" unless chltype

            "#{qmgr}/#{chltype}/#{channel}"
          end
        end
      end
    end
  end
end
