#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'
require 'puppet_x/enterprisemodules/mq/commands'
require 'puppet_x/enterprisemodules/mq/runmqsc_command'

module PuppetX
  module EnterpriseModules
    module Mq
      #
      # Define methods common for MQ types
      #
      module Type
        def self.included(parent)
          parent.send(:include, EasyType)
          parent.send(:include, PuppetX::EnterpriseModules::Mq::Commands)
          parent.extend ClassMethods
        end

        def running?(qmgr)
          self.class.running_queue_managers.include?(qmgr)
        end

        def when_running
          if running?(qmgr)
            yield
          else
            Puppet.notice "Skipping MQ config modification because on queue manager #{qmgr} is running in standby mode."
            nil
          end
        end

        # Class methods
        module ClassMethods
          #
          # Returns a Hash where the keys are the queue manager names. The content
          # is the state of the queue
          #
          def queue_managers
            info = dspmq.scan(/QMNAME\((?<queue>.*?)\)\s*STATUS\((?<status>.*?)\)/)
            info.to_h { |name, status| [name, status] }
          end

          def running_queue_managers
            @running_queue_managers ||= queue_managers.dup.delete_if { |_l, v| v != 'Running' }
            @running_queue_managers
          end

          def queues_on(qmgr)
            runmqsc('display queue(*)', :qmgr => qmgr).scan(/QUEUE\((.*?)\)\s+TYPE\((.*?)\)/).to_h
          end

          def topics_on(qmgr)
            runmqsc('display topic(*)', :qmgr => qmgr).scan(/TOPIC\((.*?)\)\s+TYPE\((.*?)\)/).to_h
          end

          def listeners_on(qmgr)
            # Take care. This one returns an array. The other ones return a Hash
            runmqsc('display listener(*)', :qmgr => qmgr).scan(/LISTENER\((\S+)\)/).flatten
          end

          def channels_on(qmgr)
            runmqsc('display channel(*)', :qmgr => qmgr).scan(/CHANNEL\((.*?)\)\s+CHLTYPE\((.*?)\)/)
          end

          # Returns `{subid: 'sub_name'}`
          def subscriptions_on(qmgr)
            runmqsc('display sub(*)', :qmgr => qmgr).scan(/SUBID\((.*?)\)\s+SUB\((.*?)\)/).to_h
          end

          def channel_authentication_records_on(qmgr)
            # return an Array and let the caller find out the exact name of the resource
            # exit code 10 means AMQ8884E: Channel authentication record not found.
            output = runmqsc('display chlauth(*) all match(all)', :qmgr => qmgr, :allowed_exits => [0, 10])
            records = output.split('Display channel authentication record details.')
            records.shift # Remove first element
            records.collect do |record|
              record_info = record.scan(/CHLAUTH\((.*?)\)\s+TYPE\((.*?)\)(?:.*ADDRESS\((.*?)\))?/m).first
              record_info << record
              record_info
            end
          end
        end
      end
    end
  end
end
